/*
 This is a Posix (i.e., Linux or Unix) HTTP client program.

*/
#include <stdio.h>      // for fprintf() and fdopen()
#include <stdlib.h>     // for atoi() and exit()
#include <string.h>     // for memset(), bzero(), bcopy(), and strerror()
#include <errno.h>      // for errno

#include <unistd.h>     // for close()
#include <sys/socket.h> // for socket(), connect(), AF_INET, SOCK_STREAM
#include <arpa/inet.h>  // for sockaddr_in, htons()
#include <netdb.h>      // for gethostbyname(), hostent, hstrerror(), and h_errno


// prototypes for two error handling functions (defined below)
void unix_error(char *msg);
void  dns_error(char *msg);


int main(int argc, char **argv)
{
   int socket_fd;                   // socket file descriptor
   struct sockaddr_in serverAddr;   // HTTP server address structure
   struct hostent *hp;              // used to get HTTP server's IP address
   char *server;                    // HTTP Server address
   char *servPort;                  // HTTP server port
   char *requestString;             // String to send to server
   #define RCVBUFSIZE 5000          // Size of response buffer
   char responseBuffer[RCVBUFSIZE]; // Buffer for response string
   int requestStringLen;            // Length of request string
   int bytesRcvd, totalBytesRcvd;   // Bytes read in single recv() and total bytes read

   server = "math.purduecal.edu";   // math.purduecal.edu server
   servPort = "80";                 // 80 is the well-known port for HTTP

   requestString = "GET /~rlkraft/roger.html HTTP/1.1\nhost:math.purduecal.edu\n\n";
   requestStringLen = strlen(requestString);

   // Create a reliable, stream socket using TCP.
   if ((socket_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      unix_error("socket() error");

   // Construct a server address structure.
   bzero(&serverAddr, sizeof(serverAddr));        // zero out the address structure
   memset(&serverAddr, 0, sizeof(serverAddr));    // zero out the address structure
   serverAddr.sin_family = AF_INET;               // Internet address family
   if ( (hp = gethostbyname( server )) == NULL )  // fill in server's IP address
      dns_error("gethostbyname() error");
   bcopy(hp->h_addr, (struct sockaddr*)&serverAddr.sin_addr, hp->h_length);
   serverAddr.sin_port = htons( atoi(servPort) ); // fill in the port number

   // Establish a connection with the server.
   if (connect(socket_fd, (struct sockaddr*)&serverAddr, sizeof(serverAddr)) < 0)
      unix_error("connect() error");


   /* Send the request string, including the null terminator, to the server */
   if (send(socket_fd, requestString, requestStringLen, 0) != requestStringLen)
      unix_error("send() sent a different number of bytes than expected");

   /* Receive the response string back from the server */
   totalBytesRcvd = 0;
   printf("Received: ");                /* Setup to print the echoed string */
   while (totalBytesRcvd < RCVBUFSIZE)
   {
      /* Receive up to the buffer size bytes from the sender
         (minus 1 to leave space for a null terminator) */
      if ((bytesRcvd = recv(socket_fd, responseBuffer, RCVBUFSIZE - 1, 0)) <= 0)
         unix_error("recv() failed or connection closed prematurely");
      totalBytesRcvd += bytesRcvd;       /* Keep tally of total bytes */
      responseBuffer[bytesRcvd] = '\0';  /* Add \0 so printf knows where to stop */
      printf(responseBuffer);            /* Print the echo buffer */
   }

   printf("\n");    /* Print a final linefeed */

   // Close the socket.
   if (close(socket_fd) < 0)
      unix_error("close() error");

   exit(0);
}


/*
   Below are two error handling functions.
   They provide reasonably meaningful error messages.
*/
void unix_error(char *msg)
{
   fprintf(stderr, "%s: %s (errno = %d)\n", msg, strerror(errno), errno);
   exit(1);
}

void dns_error(char *msg)
{
   fprintf(stderr, "%s: %s (h_errno = %d)\n", msg, hstrerror(h_errno), h_errno);
   exit(1);
}
