/*
   This is a Windows "WinSock" HTTP client program.

   If you are using gcc, be sure to use the library option
      -lwsock32
*/
#include <stdio.h>      /* for printf(), fprintf() */
#include <winsock.h>    /* for socket(),... */
#include <stdlib.h>     /* for exit() */


void DieWithError(char *errorMessage);  /* Error handling function */


int main(int argc, char *argv[])
{
   int sock;                        /* Socket descriptor */
   struct sockaddr_in servAddr;     /* HTTP server address  structure */
   char *servIP;                    /* Server IP address (dotted quad) */
   char *servPort;                  /* HTTP server port */
   char *requestString;             /* String to send to server */
   #define RCVBUFSIZE 5000          /* Size of response buffer */
   char responseBuffer[RCVBUFSIZE]; /* Buffer for response string */
   int requestStringLen;            /* Length of request string */
   int bytesRcvd, totalBytesRcvd;   /* Bytes read in single recv() and total bytes read */
   WSADATA wsaData;                 /* Structure for WinSock setup communication */

   servIP = "205.215.68.105";       /* math.purduecal.edu server IP address (dotted quad) */
   servPort = "80";                 /* 80 is the well-known port for HTTP */

   requestString = "GET /~rlkraft/roger.html HTTP/1.1\nhost:math.purduecal.edu\n\n";
   requestStringLen = strlen(requestString);

   if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0) /* Load Winsock 2.0 DLL */
   {
      fprintf(stderr, "WSAStartup() failed");
      exit(1);
   }

   /* Create a reliable, stream socket using TCP */
   if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
      DieWithError("socket() failed");

   /* Construct the server address structure */
   memset(&servAddr, 0, sizeof(servAddr));           /* Zero out structure */
   servAddr.sin_family      = AF_INET;               /* Internet address family */
   servAddr.sin_addr.s_addr = inet_addr(servIP);     /* Server IP address */
   servAddr.sin_port        = htons(atoi(servPort)); /* Server port */

   /* Establish the connection to the echo server */
   if (connect(sock, (struct sockaddr *) &servAddr, sizeof(servAddr)) < 0)
      DieWithError("connect() failed");


   /* Send the request string, including the null terminator, to the server */
   if (send(sock, requestString, requestStringLen, 0) != requestStringLen)
      DieWithError("send() sent a different number of bytes than expected");

   /* Receive the response string back from the server */
   totalBytesRcvd = 0;
   printf("Received: ");                /* Setup to print the echoed string */
   while (totalBytesRcvd < RCVBUFSIZE)
   {
      /* Receive up to the buffer size bytes from the sender
         (minus 1 to leave space for a null terminator) */
      if ((bytesRcvd = recv(sock, responseBuffer, RCVBUFSIZE - 1, 0)) <= 0)
         DieWithError("recv() failed or connection closed prematurely");
      totalBytesRcvd += bytesRcvd;       /* Keep tally of total bytes */
      responseBuffer[bytesRcvd] = '\0';  /* Add \0 so printf knows where to stop */
      printf(responseBuffer);            /* Print the echo buffer */
   }

   printf("\n");    /* Print a final linefeed */

   // Close the socket.
   closesocket(sock);
   WSACleanup();  /* Cleanup Winsock */

   exit(0);
}


/*
   Below is an error handling function.
   It provides reasonably meaningful error messages.
*/
void DieWithError(char *errorMessage)
{
   fprintf(stderr,"%s: %d\n", errorMessage, WSAGetLastError());
   exit(1);
}