/*
   A simple HTTP client.

   This program gets its URL
   from the command line.

   Notice that a command like
      >java HTTPClient_v3 www.google.com
   does not work. You need to make sure that the URL
   has a "path" in it. So the command should look like
   this.
      >java HTTPClient_v3 www.google.com/
*/
import java.net.*;
import java.io.*;

public class HTTPClient_v3
{
   public static void main(String[] args) throws IOException
   {
      if (0 == args.length)
      {
         System.err.println("Usage: java HTTPClient_v3 <url>");
         System.exit(1);
      }

      // Create a URL object using the command line argument.
      URL url = new URL( "http://" + args[0] );  // always use HTTP

      // Set the port number.
      int servPort = 80;

      // Set the message for the server (using parts of the URL).
      String message = "GET " + url.getPath() + " HTTP/1.1\r\nhost:" + url.getHost() + "\r\n\r\n";


      // Create a Socket object that is connected to the server on the specified port.
      Socket socket = new Socket(url.getHost(), servPort);
      System.out.println("Connected to server...sending request:\n");

      // Get (low-level) input and output streams from the socket.
      InputStream   in = socket.getInputStream();
      OutputStream out = socket.getOutputStream();

      // Make the low-level input and output streams easier to use.
      BufferedReader br = new BufferedReader( new InputStreamReader(in, "latin1") );
      PrintStream    ps = new PrintStream( out );

      // Send the message to the server.
      ps.print(message);
      ps.flush();

      // Receive the response from the server  (including the "response headers").
      String oneLine;
      while ( (oneLine = br.readLine()) != null )
      {
         System.out.println(oneLine);
      }

      socket.close();  // Close the socket and its streams

      System.out.println( "\nconnection closed:" );
   }
}