/*
   This is an simple client that reads lines from standard input,
   writes each line to the server, reads back the response line
   from the server, and then prints that response line to
   standard output. This client does not close its connection to
   the server until it sees that its standard input has been closed.
*/
import java.net.*;
import java.io.*;
import java.util.Scanner;

public class TCPFilterClient
{
   public static void main(String[] args) throws IOException
   {
      if (0 == args.length) // check for command line arguments
      {
         System.err.println("Usage: java TCPFilterClient <Server> [<Port>]");
         System.exit(1);
      }

      String server = args[0];  // Server name or IP address
      int servPort = (2 == args.length) ? Integer.parseInt(args[1]) : 7;

      // Create a socket that connects to the server on the specified port.
      Socket socket = new Socket(server, servPort);

      // Information about the connection that was just established.
      System.err.println( "Connected to server " +
         socket.getInetAddress().getCanonicalHostName() + " at " +
            socket.getInetAddress().getHostAddress() + ":" + socket.getPort() +
               " using local port " + socket.getLocalPort() );

      // Get (low-level) input and output streams from the socket.
      InputStream   in = socket.getInputStream();
      OutputStream out = socket.getOutputStream();

      // Make the low-level input and output streams easier to use.
      BufferedReader br = new BufferedReader( new InputStreamReader(in, "latin1") );
      PrintStream    ps = new PrintStream( out );

      // Create a Scanner object to make it easier to use System.in
      Scanner scanner = new Scanner( System.in );

      // Read from stdin, send to server, get response, write it to stdout.
      while ( scanner.hasNextLine() )
      {
         String oneLine = scanner.nextLine();  // read a line from stdin
         ps.println( oneLine );                // send it to the server
         //ps.flush();
         oneLine = br.readLine();              // get the echo back
         System.out.println( oneLine );        // write to stdout
      }

      socket.close();  // Close the socket and its streams
   }
}