/*
   This server is more or less an echo server but it filters
   what it echos (it converts to upper case). This server
   will continue to echo to the client until the client
   closes its side of the connection.

   Try running two instances of this server on the same port
   number. What happens? Try running two instances of this
   server on two different port numbers.

   Try running this server and its client on two different
   computers at the same time, so you are  really using a
   network.

   Use the Windows command line program netstat to
   get information about this server and its clients.
      C:\>netstat -n -p tcp
      C:\>netstat -a -n -p tcp
      C:\>netstat /?
*/
import java.net.*;
import java.io.*;

public class TCPFilterServer
{
   public static void main(String[] args) throws IOException
   {
      if (0 == args.length) // check for a command line argument
      {
         System.err.println("Usage: java TCPFileterServer <Port>");
         System.exit(1);
      }

      // Get the server port number from the command line parameter.
      int servPort = Integer.parseInt(args[0]);

      // Create a server socket to accept client connection requests.
      ServerSocket servSock = new ServerSocket(servPort);
      // This server socket is not yet in an accepting state, so
      // clients cannot yet connect to this server.

      while (true) // Run forever, accepting and servicing connections.
      {
         System.err.println("Server waiting for a connection...");

         // Wait for a client connection.
         Socket clntSock = servSock.accept();

         // When accept() returns, that means we are connected, on
         // a new(!) socket, to a client process.

         // Information about the connection that was just established.
         System.err.print( "Handling client " +
            clntSock.getInetAddress().getCanonicalHostName() + " at " +
               clntSock.getInetAddress().getHostAddress() + ":" + clntSock.getPort() +
                  " using local port " + clntSock.getLocalPort() + "..." );

         // Get (low-level) input and output streams from the socket.
         InputStream   in = clntSock.getInputStream();
         OutputStream out = clntSock.getOutputStream();

         // Make the low-level input and output streams easier to use.
         BufferedReader br = new BufferedReader( new InputStreamReader(in, "latin1") );
         PrintStream    ps = new PrintStream( out );

         // Receive (and echo) until the client closes the connection.
         String oneLine;
         while ( (oneLine = br.readLine()) != null )
         {
            ps.println( oneLine.toUpperCase() ); // echo the line back to the client
            //ps.flush();
         }

         System.err.println( "Closing connection " +
            clntSock.getInetAddress().getHostAddress() + ":" + clntSock.getPort() );

         clntSock.close();  // Close the socket.  We are done with this client!
      }
   }
}