/*
   This program provides a "filter service". This "server" is
   more or less an echo server but it filters what it echoes
   (it converts to upper case).

   Try running two instances of this service on the same port
   number. What happens? Try running an instance of this udp
   service and an instance of the tcp server using the same
   port number.

   Try running this service and several clients simultaneously.
   What happens? How does this differ from the tcp server?

   Use the Windows command line program netstat to
   get information about this service and its clients.
      C:\>netstat -n -p udp
      C:\>netstat -a -n -p udp
      C:\>netstat /?
*/
import java.net.*;
import java.io.*;

public class UDPFilterService
{
   private static final int DATAMAX = 255;  // Maximum size of datagram

   public static void main(String[] args) throws IOException
   {
      if (0 == args.length) // check for a command line argument
      {
         System.err.println("Usage: java UDPFilterService <Port>");
         System.exit(1);
      }

      // Get the service port number from the command line parameter.
      int servPort = Integer.parseInt(args[0]);

      // Create a datagram (udp) socket to receive and send packets with.
      // The socket is bound to the given port number (which "clients" should know).
      DatagramSocket servSock = new DatagramSocket(servPort);
      // This socket is not yet in a receiving state,
      // so clients cannot yet send packets to it.

      DatagramPacket packet = new DatagramPacket(new byte[DATAMAX], DATAMAX);

      while (true) // Run forever, accepting and servicing packets.
      {
         System.err.print("Server waiting for a packet...");

         // Receive a packet from a client
         servSock.receive(packet);

         // When receive() returns, we have received a packet from a client process.

         // Information about the packet that was just received.
         System.err.println( "received on port " +
            servSock.getLocalPort() + " from " +
               packet.getAddress().getHostAddress() + ":" + packet.getPort() );

         // Information about the data in the received packet.
         System.err.println("\tpacket.getLength()      = " + packet.getLength() );
         System.err.println("\tpacket.getData().length = " + packet.getData().length );

         // Filter the data in the packet's byte array.
         byte[] data    = packet.getData();
         int dataLength = packet.getLength();
         for (int i = 0; i < dataLength; i++)
         {
            data[i] = (byte)Character.toUpperCase( (char)data[i] );
         }

         // Log the returned data to the console.
         System.err.println( new String(data, 0, packet.getLength()) );
         System.err.println( new String(data, 0, packet.getData().length).trim() );

         servSock.send(packet);     // Send the packet back to the client.
         packet.setLength(DATAMAX); // Reset length to avoid shrinking buffer.
                                    // (You don't want the receive() method to
                                    // only receive as many bytes as the length
                                    // of the last packet's data.)
      }
   }
}