/*
   A template for TCP clients.
   This client does not implement any application layer protocol.

   Use the Windows command line program netstat to
   get information about this client and its server.
      C:\>netstat -n -p tcp
      C:\>netstat -a -n -p tcp
      C:\>netstat /?
*/
import java.net.*;
import java.io.*;
import java.util.Scanner;

public class TCPClientTemplate
{
   public static void main(String[] args) throws IOException
   {
      // Check for two command line arguments.
      if (2 > args.length)
      {
         System.err.println("Usage: java TCPClientTemplate <Server> <Port>");
         System.exit(1);
      }

      String server = args[0];                  // Server's name or IP address.
      int servPort = Integer.parseInt(args[1]); // Server's port number.

      // Create a socket that connects to the server on the specified port.
      Socket socket = new Socket(server, servPort);

      // Information about the connection that was just established.
      System.err.println( "Connected to server " +
         socket.getInetAddress().getCanonicalHostName() + " at " +
            socket.getInetAddress().getHostAddress() + ":" + socket.getPort() +
               " using local port " + socket.getLocalPort() );

      // Get (low-level) input and output streams from the socket.
      InputStream   in = socket.getInputStream();
      OutputStream out = socket.getOutputStream();

      // Make the low-level input and output streams easier to use.
      Scanner     sc = new Scanner( new InputStreamReader(in, "latin1") );
      PrintStream ps = new PrintStream( out );

      // Notice that this client now has five open streams.
      // stdin     read-only
      // stdout    write-only
      // stderr    write-only
      // sc        read-only
      // ps        write-only


      // Implement an application level protocol.

      // NOTE: Who "talks" first, the client or the server,
      // is determined by the application level protocol.




      System.err.println( "Closing connection " +
         socket.getInetAddress().getHostAddress() + ":" + socket.getPort() );

      ps.close();      // Close the streams to the socket.
      sc.close();
      socket.close();  // Close the socket.
      // NOTE: Who closes their socket first, the client or the server,
      // is also determined by the application level protocol.
   }
}