/*
   A template for TCP (serial) servers.
   This server does not implement any application layer protocol.

   Use the Windows command line program netstat to
   get information about this server and its clients.
      C:\>netstat -n -p tcp
      C:\>netstat -a -n -p tcp
      C:\>netstat /?
*/
import java.net.*;
import java.io.*;
import java.util.Scanner;

public class TCPServerTemplate
{
   public static void main(String[] args) throws IOException
   {
      // Check for a command line argument.
      if (0 == args.length)
      {
         System.err.println("Usage: java TCPServerTemplate <Port>");
         System.exit(1);
      }

      // Get the server port number from the command line parameter.
      int servPort = Integer.parseInt(args[0]);

      // Create a server socket to accept client connection requests.
      ServerSocket servSock = new ServerSocket(servPort);
      // This server socket is not yet in an accepting state, so
      // clients cannot yet connect to this server.

      while (true) // Run forever, accepting and servicing connections.
      {
         System.err.println("Server waiting for a connection...");

         // Wait for a client connection.
         Socket clntSock = servSock.accept();

         // When accept() returns, that means we are connected, on
         // a new(!) communicating socket, to a client process.

         // Information about the connection that was just established.
         System.err.print( "Handling client " +
            clntSock.getInetAddress().getCanonicalHostName() + " at " +
               clntSock.getInetAddress().getHostAddress() + ":" + clntSock.getPort() +
                  " using local port " + clntSock.getLocalPort() + "..." );

         // Get (low-level) input and output streams from the socket.
         InputStream   in = clntSock.getInputStream();
         OutputStream out = clntSock.getOutputStream();

         // Make the low-level input and output streams easier to use.
         Scanner     sc = new Scanner( new InputStreamReader(in, "latin1") );
         PrintStream ps = new PrintStream( out );

         // Notice that this server now has five open streams.
         // stdin     read-only
         // stdout    write-only
         // stderr    write-only
         // sc        read-only
         // ps        write-only


         // Implement an application level protocol.

         // NOTE: Who "talks" first, the server or the client,
         // is determined by the application level protocol.




         System.err.println( "Closing connection " +
            clntSock.getInetAddress().getHostAddress() + ":" + clntSock.getPort() );

         ps.close();       // Close the streams to the communicating socket.
         sc.close();
         clntSock.close(); // Close communicating socket. We are done with this client!
         // NOTE: Who closes their socket first, the server or the client,
         // is also determined by the application level protocol.
      }
   }
}