/*
   adderGET.c - a minimal CGI program that adds two numbers together

   This program handles two kinds of URLs.
   The HTML form type URL with key=value, e.g.,
      cgi-bin/adderGET.exe?x=5&y=9
   And a non standard type of URL that only has values, e.g.,
      cgi-bin/adderGET.exe?5&9

   NOTE: For a cgi process, standard output has been redirected to a socket.
         So printf() in this process will be sending data over a socket!
         What if the other end of the socket is closed prematurely? Then
         printf() could generate a SIGPIPE signal. Or printf() could
         return with an EPIPE error (or error numbers 104 or 113). For a
         simple cgi process, it is probably OK to just let the process die.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define  MAXLINE   8192  /* max text line length */

int main(void)
{
   char *buf, *p;
   char arg1[MAXLINE], arg2[MAXLINE], content[MAXLINE];
   int n1=0, n2=0;

   /* Extract the two arguments from QUERY_STRING */
   if ((buf = getenv("QUERY_STRING")) != NULL)
   {
      p = strchr(buf, '&');
      *p = '\0';
      strcpy(arg1, buf);
      strcpy(arg2, p+1);
      *p = '&';

      p = strchr(arg1, '=');
      if (!p)
      {
         n1 = atoi(arg1);
      }
      else  // the URL is of the form key=value
      {
         n1 = atoi(++p);
      }

      p = strchr(arg2, '=');
      if (!p)
      {
         n2 = atoi(arg2);
      }
      else  // the URL is of the form key=value
      {
         n2 = atoi(++p);
      }
   }

   /* Make the response body */
   sprintf(content, "Welcome to add.com: ");
   sprintf(content, "%sTHE Internet addition portal.\n<p>", content);
   sprintf(content, "%sThe answer is: %d + %d = %d\n<p>", content, n1, n2, n1 + n2);
   sprintf(content, "%sThanks for visiting!\n", content);

   /* Send the HTTP response headers */
   printf("Content-type: text/html\n");
   printf("Content-length: %d\n", strlen(content));
   printf("\n");

   /* Send the response body */
   printf("%s", content);
   fflush(stdout);

   // simulate a complicated cgi program
 //int i;
 //double x;
 //for (i=0; i <= 1000000000; i++) x = ((double)i)*i;  // waste time

   exit(0);
}