/*
   adderAjax_win32.c - a minimal CGI program that adds two numbers together

   This version of adder can handle either a GET or a POST request method.
   This program reads the CGI environment variable REQUEST_METHOD to
   determine which kind of request it should handle.

   This program handles two kinds of query strings (for both GET and POST).
   The HTML form type query string, with key=value, e.g.,
      cgi-bin/adder.exe?x=5&y=9
   And a non-standard type of query string that only has values, e.g.,
      cgi-bin/adder.exe?5&9

   NOTE: For a cgi process, standard output has been redirected to a socket.
         So printf() in this process will be sending data over a socket!
         What if the other end of the socket is closed prematurely? Then
         printf() could generate a SIGPIPE signal. Or printf() could
         return with an EPIPE error (or error numbers 104 or 113). For a
         simple cgi process, it is probably OK to just let the process die.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>

#define  MAXLINE   8192  /* max text line length */

int main(void)
{
   char *buf1, *buf2, *len, *p;
   char buf3[MAXLINE], arg1[MAXLINE], arg2[MAXLINE], content[MAXLINE];
   int n1=0, n2=0;
   int length=-1, i=0;

   /* Get the request method. */
   if ( (buf1 = getenv("REQUEST_METHOD")) != NULL )
   {
      if ( 0 == strcmp(buf1, "GET") )
      {
         /* Get the query string from an environment variable. */
         if ( (buf2 = getenv("QUERY_STRING")) != NULL )
         {
            /* Extract the two arguments from query string */
            p = strchr(buf2, '&');
            *p = '\0';
            strcpy(arg1, buf2);
            strcpy(arg2, p+1);
            *p = '&';

            p = strchr(arg1, '=');
            if (!p)
            {
               n1 = atoi(arg1);
            }
            else  /* the query string is of the form key=value */
            {
               n1 = atoi(++p);
            }

            p = strchr(arg2, '=');
            if (!p)
            {
               n2 = atoi(arg2);
            }
            else  /* the query string is of the form key=value */
            {
               n2 = atoi(++p);
            }
         }
      }
      else if ( 0 == strcmp(buf1, "POST") )
      {
         /* Get the query string from standard input. */
         if ((len = getenv("CONTENT_LENGTH")) != NULL)
         {
            length = atoi(len);
            for (i = 0; i < length; i++)
               buf3[i] = getchar();  /* read query string from stdin */
            buf3[length] = 0;        /* null terminate the query string */

            /* Extract the two arguments from the query string. */
            p = strchr(buf3, '&');
            *p = '\0';
            strcpy(arg1, buf3);
            strcpy(arg2, p+1);
            *p = '&';

            p = strchr(arg1, '=');
            if (!p)
            {
                n1 = atoi(arg1);
            }
            else  /* the query string is of the form key=value */
            {
               n1 = atoi(++p);
            }

            p = strchr(arg2, '=');
            if (!p)
            {
               n2 = atoi(arg2);
            }
            else  /* the query string is of the form key=value */
            {
               n2 = atoi(++p);
            }
         }
      }
      else  /* Wrong kind of request method. */
      {
      }
   }

   /* Make the response body */
   sprintf(content, "%d", n1 + n2);

   /* Send the HTTP response headers */
   printf("Content-type: text/html\n");
   printf("Content-length: %d\n", strlen(content));
   printf("\n");

   /* Send the response body */
   printf("%s", content);
   fflush(stdout);

   Sleep(180000);  // Three minute pause. This gives you time to look for
                   // this process using ProcessHacker. You can see that it
                   // is a child of the web server, and you can examine its
                   // environment variables and look for the CGI variables.
   exit(0);
}